#include "global.h"

/*

object3d.cc - load and display a Jake format file (.jfm)
Copyright (c) 2006 Jake Stookey

*/


Object3d::Object3d() {
  ttexture1 = TEX_NONE;
  currentframe = 0;
}

Object3d::Object3d(int tex_texture1, char *filename) {
  ttexture1 = tex_texture1;
  currentframe = 0;

  // Load the object file

  printf("Loading Object3d: %s\n", filename);

  FILE *fp = fopen(filename, "r");
  if (fp == NULL) {
    printf("Error opening output file!\n");
  } 

  // Read 4 ints from data file
  // num vertices
  fread(&nv, sizeof(int), 1, fp);
  fread(&num_vertex_indices, sizeof(int), 1, fp);
  // Num texture coordinates
  fread(&nt, sizeof(int), 1, fp);
  fread(&num_texture_indices, sizeof(int), 1, fp);
  // Num frames in animation
  fread(&num_frames, sizeof(int), 1, fp);

  // Allocate space for the arrays
  // allocate vertices for each animation frame
  vertices = (float *)malloc(sizeof (float) * nv * num_frames);
  vertexindices = (int *)malloc(sizeof (int) * num_vertex_indices);
  texturecoords = (float *)malloc(sizeof (float) * nt);
  textureindices = (int *)malloc(sizeof (int) * num_texture_indices);

  // Read arrays from data file
  fread(vertices, sizeof(float), nv * num_frames, fp);
  fread(vertexindices, sizeof(int), num_vertex_indices, fp);
  fread(texturecoords, sizeof(float), nt, fp);
  fread(textureindices, sizeof(int), num_texture_indices, fp);

  fclose(fp);
}

Object3d::~Object3d() {
  // Hm, I should probably deallocate stuff here
}

void Object3d::draw() {
  int i = 0, j = 0, k;
  int num_verts = 0;

  int mode = GL_TRIANGLES; // GL_QUADS or GL_TRIANGLES
  int prev_mode = GL_TRIANGLES;

  // Enable texture mapping if there's a tecture associated with the object
  if (ttexture1) {
    glBindTexture(GL_TEXTURE_2D, texture[ttexture1]);
    glEnable(GL_TEXTURE_2D);
  }

  glPushMatrix();

  glBegin(GL_TRIANGLES);  // Initially assume tri's and change it later if need be

  // Draw all the vertices
  while (i < num_vertex_indices) {
    // Determine mode  (quads or triangles)
    if (vertexindices[i+3] == -1) {
      mode = GL_TRIANGLES;
      num_verts = 3;
    } else if (vertexindices[i+4] == -1) {
      mode = GL_QUADS;
      num_verts = 4;
    } else {
      printf("Unexpected error detecting mode - quads or tri's, i: %i, %i\n", i, vertexindices[i+3]);
    }

    // If mode has changed, glEnd(); glBegin(mode)
    if (mode != prev_mode) {
      //Change mode!
      glEnd();
      glBegin(mode);
    }

    // Draw the vertex and texture
    for (k = 0; k < num_verts; k++) {
      if (num_texture_indices != 0) glTexCoord2f(texturecoords[(textureindices[i]*2)], texturecoords[(textureindices[i]*2 + 1)]);
      glVertex3f(vertices[(vertexindices[i]*3)  + (currentframe * nv)], vertices[(vertexindices[i]*3+1)  + (currentframe * nv)], vertices[(vertexindices[i]*3+2)  + (currentframe * nv)]);
      i++;
    }

    i++; // One more increment for the "-1" value
    prev_mode = mode;
  }
  glEnd(); 

  glPopMatrix();
  if (ttexture1) glDisable(GL_TEXTURE_2D);

  // Advance the animation frame for next time
  currentframe = (currentframe + 1) % num_frames;
}
