#include "global.h"

/*

kglgame.cc
Copyright (c) 2006 Jake Stookey

*/

#ifdef DREAMCAST
  extern uint8 romdisk[];
  KOS_INIT_ROMDISK(romdisk);
  #define FILE_NAME(x) "/rd/" x
#else
  #define GL_TEXTURE_FILTER 0
  #define GL_FILTER_BILINEAR 0
  #define GL_MODULATEALPHA 0
  #include "pcx.c"
  #define FILE_NAME(x) "romdisk/" x
#endif

float yrot = 0.0f;
float xrot = 0.0f;

GLuint texture[TEX_NUM_TEXTURES + 10];

Object3d *o3d_dreamcast;

int analogx = 0;
int analogy = 0;

void draw_gl(void) {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glPushMatrix();
  glTranslatef(0.0f,0.0f,-10.0f);
  // Rotation based on user input
  glRotatef(-xrot, 0.0f, 1.0f, 0.0f);
  glRotatef(-yrot, 1.0f, 0.0f, 0.0f);
  glRotatef(80, 1.0f, 0.0f, 0.0f);
  o3d_dreamcast->draw();
  glPopMatrix();
}

int check_input() {
#ifdef DREAMCAST
  int	c;

  c = maple_first_controller();

  cont_cond_t cond;
    
  /* Check key status */
  if (cont_get_cond(c, &cond) < 0) {
    printf("Error reading controller\n");
    return (-1);
  }
  
  if (!(cond.buttons & CONT_START))
    return(-1);
  
  if (!(cond.buttons & CONT_A)) {
    pvr_stats_t     stats;
    pvr_get_stats(&stats);
    printf("VBL Count: %d, last_time: %f, frame rate: %f fps\n",
	   stats.vbl_count, stats.frame_last_time, stats.frame_rate);
  }

  analogx = cond.joyx;
  analogy = cond.joyy;
#endif
}

void draw_scene() {
#ifdef DREAMCAST
  glKosBeginFrame();    
  draw_gl();
  glKosFinishFrame();
#else
  draw_gl();
  glFlush();
  glutSwapBuffers();
#endif
}

void get_rotations() {
  yrot = (128 - analogy)*(90.0/128.0);
  xrot = (128 - analogx)*(90.0/128.0);
}

#ifdef DREAMCAST
#else
static void timerfunc(int value) {
  get_rotations();
  draw_scene();
  glutTimerFunc(10, timerfunc, 1);
  
}

static void motionfunc(int x, int y) {
  analogx = x;
  analogy = y;
}

#endif

void init(int argc, char **argv) {
  #ifdef DREAMCAST
  init_kos();
  #else
  glutInit(&argc,argv);

  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_ALPHA);
  glutInitWindowSize(640,480);
  glutInitWindowPosition(0, 0);

  glutCreateWindow("kglgame");
  glutDisplayFunc(draw_scene);
  glutTimerFunc (10, timerfunc, 1);
  glutPassiveMotionFunc(motionfunc);

  #endif
}

void main_loop() {

#ifdef DREAMCAST
  while(1) {
    if (check_input() == -1) {
      break;
    }
    
    get_rotations();
    
    draw_scene();    
  }
#else
  glutMainLoop();
#endif
}

int main(int argc, char **argv) {	
  int i;
  xrot = 0.0f;
  yrot = 0.0f;

  init(argc,argv);

  o3d_dreamcast = new Object3d(TEX_DREAMCAST, FILE_NAME("dreamcast.jfm")); 
  loadtxr( FILE_NAME("dreamcast.pcx"), &texture[TEX_DREAMCAST]);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0f,640.0f/480.0f,0.1f,100.0f);
  glMatrixMode(GL_MODELVIEW);
  
  glFrontFace(GL_CW);
  /* Enable Transparancy */
  glColor4f(1.0f, 1.0f, 1.0f, 0.5f);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_FILTER, GL_FILTER_BILINEAR);
  glTexEnvi(GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATEALPHA);

  glLoadIdentity();
  
  glShadeModel(GL_FLAT);
  glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
  glClearDepth(1.0f);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);
  
  glLoadIdentity();
  
  /******  MAIN LOOP *****/
  main_loop();

  return 0;
}
